@@ -2,11 +2,12 @@ from netCDF4 import Dataset, numpy
 import numpy as np
 import os
 from glob import glob
+from contextlib import contextmanager
 
 
-def flatten(x):
+def flatten(lst):
     result = []
-    for el in x:
+    for el in lst:
         if hasattr(el, "__iter__") and not isinstance(el, str):
             result.extend(flatten(el))
         else:
@@ -14,29 +15,29 @@ def flatten(x):
     return result
 
 
-dtypes = {}
-dtypes[numpy.dtype('float32')] = 'f4'
-dtypes[numpy.dtype('int32')] = 'i4'
-dtypes[numpy.dtype('int8')] = 'i1'
-dtypes[numpy.dtype('S1')] = 'S1'
+DTYPES = {}
+DTYPES[numpy.dtype('float32')] = 'f4'
+DTYPES[numpy.dtype('int32')] = 'i4'
+DTYPES[numpy.dtype('int8')] = 'i1'
+DTYPES[numpy.dtype('S1')] = 'S1'
 
 
 class NCObject(object):
 
     @classmethod
-    def open(self, files_or_pattern):
-        files, pattern = self.distill(files_or_pattern)
-        o = self.choice_type(files)
-        o.pattern = pattern
-        o.load()
-        return o
+    def open(cls, files_or_pattern):
+        files, pattern = cls.distill(files_or_pattern)
+        obj = cls.choice_type(files)
+        obj.pattern = pattern
+        obj.load()
+        return obj
 
     @classmethod
-    def choice_type(self, files):
+    def choice_type(cls, files):
         return NCPackage(files) if len(files) > 1 else NCFile(files)
 
     @classmethod
-    def distill(self, files_or_pattern):
+    def distill(cls, files_or_pattern):
         if files_or_pattern in ['', []]:
             raise Exception('There is not file list or '
                             'pattern to open.')
@@ -60,6 +61,7 @@ class NCObject(object):
         self.roots = []
         self.variable_wrapper = lambda name, vars: name, vars
         self.create_dim = 'create_dimension'
+        self._read_only = True
 
     @property
     def is_new(self):
@@ -86,18 +88,18 @@ class NCObject(object):
                 if self.has_dimension(name)
                 else self.create_dimension(name, size))
 
-    def obtain_variable(self, name, vtype='f4', dimensions=(), digits=0,
-                        fill_value=None):
-        raise Exception('Subclass responsability')
+    def obtain_variable(self, *args, **kwargs):
+        raise Exception('Subclass responsability (should process %s and %s)' %
+                        (str(args), str(kwargs)))
 
     def getvar(self, name, vtype='', dimensions=(), digits=0,
                fill_value=None, source=None):
         if source:
             self.copy_in(name, vtype, source)
         if name not in self.variables.keys():
-            vars = self.obtain_variable(name, vtype, dimensions,
-                                        digits, fill_value)
-            self.variables[name] = self.variable_wrapper(name, vars)
+            varstmp = self.obtain_variable(name, vtype, dimensions,
+                                           digits, fill_value)
+            self.variables[name] = self.variable_wrapper(name, varstmp)
         return self.variables[name]
 
     def sync(self):
@@ -110,14 +112,15 @@ class NCObject(object):
         # create dimensions if not exists.
         dims = source.dimensions
         gt1_or_none = lambda x: len(x) if len(x) > 1 else None
-        [self.getdim(d, gt1_or_none(dims[d])) for d in dims]
+        create_dim = lambda d: self.getdim(d, gt1_or_none(dims[d]))
+        map(create_dim, dims)
         dimensions = tuple(reversed([str(k)
                                      for k in source.dimensions.keys()]))
-        vt = vtype if vtype else source.vtype
+        vtype_tmp = vtype if vtype else source.vtype
         options = {'fill_value': 0.0}
-        if vt == 'f4':
+        if vtype_tmp == 'f4':
             options['digits'] = source.least_significant_digit
-        var = self.getvar(name, vt, dimensions, **options)
+        var = self.getvar(name, vtype_tmp, dimensions, **options)
         var[:] = source[:]
 
 
@@ -125,17 +128,20 @@ class NCFile(NCObject):
 
     def load(self):
         filename = self.files[0]
-        self.read_only = True
         try:
             self.roots = [(Dataset(filename, mode='w', format='NETCDF4')
-                          if self.is_new else Dataset(filename, mode='a',
-                                                      format='NETCDF4'))]
-            self.read_only = False
+                           if self.is_new else Dataset(filename, mode='a',
+                                                       format='NETCDF4'))]
+            self._read_only = False
         except Exception:
             self.roots = [Dataset(filename, mode='r', format='NETCDF4')]
         self.variable_wrapper = SingleNCVariable
         self.create_dim = 'createDimension'
 
+    @property
+    def read_only(self):
+        return self._read_only
+
     def obtain_variable(self, name, vtype='f4', dimensions=(), digits=0,
                         fill_value=None):
         root = self.roots[0]
@@ -150,7 +156,10 @@ class NCFile(NCObject):
                    'fill_value': fill_value}
         if digits > 0:
             options['least_significant_digit'] = digits
-        return [build(name, vtype, dimensions, **options)]
+        varstmp = [build(name, vtype, dimensions, **options)]
+        not_auto_mask = lambda v: v.set_auto_maskandscale(False)
+        map(not_auto_mask, varstmp)
+        return varstmp
 
 
 class NCPackage(NCObject):
@@ -191,9 +200,9 @@ class NCVariable(object):
 
     @property
     def least_significant_digit(self):
-        v = self.variables[0]
-        return (v.least_significant_digit
-                if hasattr(v, 'least_significant_digit') else 0)
+        variable = self.variables[0]
+        return (variable.least_significant_digit
+                if hasattr(variable, 'least_significant_digit') else 0)
 
     @property
     def dtype(self):
@@ -201,7 +210,7 @@ class NCVariable(object):
 
     @property
     def vtype(self):
-        return dtypes[np.dtype(self.dtype)]
+        return DTYPES[np.dtype(self.dtype)]
 
     def __getitem__(self, indexes):
         return self.pack().__getitem__(indexes)
@@ -209,11 +218,12 @@ class NCVariable(object):
     def __getattr__(self, name):
         print 'Unhandled [class: %s, instance: %s, attr: %s]' % (
             self.__class__, self.name, name)
-        import ipdb; ipdb.set_trace()
+        import ipdb
+        ipdb.set_trace()
 
     def sync(self):
-        for v in self.variables:
-            v.group().sync()
+        for variable in self.variables:
+            variable.group().sync()
 
 
 class SingleNCVariable(NCVariable):
@@ -222,10 +232,10 @@ class SingleNCVariable(NCVariable):
         return self.variables[0].group()
 
     def pack(self):
-        vars = self.variables[0]
+        varstmp = self.variables[0]
         if self.variables[0].shape[0] > 1:
-            vars = np.vstack([self.variables])
-        return vars
+            varstmp = np.vstack([self.variables])
+        return varstmp
 
     def __setitem__(self, indexes, changes):
         return self.variables[0].__setitem__(indexes, changes)
@@ -234,14 +244,14 @@ class SingleNCVariable(NCVariable):
 class DistributedNCVariable(NCVariable):
 
     def pack(self):
-        return np.vstack([v.pack() for v in self.variables])
+        return np.vstack([variable.pack() for variable in self.variables])
 
     def __setitem__(self, indexes, change):
         pack = self.pack()
         pack.__setitem__(indexes, change)
-        vars = np.vsplit(pack, pack.shape[0])
-        for i in range(len(vars)):
-            self.variables[i][:] = vars[i]
+        varstmp = np.vsplit(pack, pack.shape[0])
+        for i in range(len(varstmp)):
+            self.variables[i][:] = varstmp[i]
         self.sync()
 
 
@@ -279,7 +289,7 @@ def getvar(root, name, vtype='', dimensions=(), digits=0, fill_value=None,
     root -- the root descriptor returned by the 'open' function
     name -- the name of the variable
     vtype -- the type of each value, ex ['f4', 'i4', 'i1', 'S1'] (default '')
-    dimensions -- the tuple with the dimension names of the variable (default ())
+    dimensions -- the tuple with dimensions name of the variables (default ())
     digits -- the precision required when using a 'f4' vtype (default 0)
     fill_value -- the initial value used in the creation time (default None)
     source -- the source variable to be copied (default None)
@@ -305,3 +315,10 @@ def close(root):
     root -- the root descriptor returned by the 'open' function
     """
     root.close()
+
+
+@contextmanager
+def loader(pattern):
+    root, _ = open(pattern)
+    yield root
+    close(root)
